@@ -1,5 +1,6 @@
 from datetime import datetime
 import os
+import asyncio
 from typing import Dict, Any, List, Optional, Callable
 from AgentCrew.modules.llm.base import BaseLLMService
 from AgentCrew.modules.llm.message import MessageTransformer
@@ -47,6 +48,7 @@ class LocalAgent(BaseAgent):
         self.registered_tools = (
             set()
         )  # Set of tool names that are registered with the LLM
+        self.is_tool_ready = True
 
     def _extract_tool_name(self, tool_def: Any) -> str:
         """
@@ -73,12 +75,26 @@ class LocalAgent(BaseAgent):
         Register tools for this agent using the services dictionary.
         """
 
-        if self.services.get("agent_manager") and not self.is_remoting_mode:
-            from AgentCrew.modules.agents.tools.transfer import (
-                register as register_transfer,
+        if self.services.get("agent_manager"):
+            from AgentCrew.modules.agents.tools.delegate import (
+                register as register_delegate,
+                delegate_tool_prompt,
             )
 
-            register_transfer(self.services["agent_manager"], self)
+            register_delegate(self.services["agent_manager"], self)
+            self.tool_prompts.append(
+                delegate_tool_prompt(self.services["agent_manager"])
+            )
+            if not self.is_remoting_mode:
+                from AgentCrew.modules.agents.tools.transfer import (
+                    register as register_transfer,
+                    transfer_tool_prompt,
+                )
+
+                register_transfer(self.services["agent_manager"], self)
+                self.tool_prompts.append(
+                    transfer_tool_prompt(self.services["agent_manager"])
+                )
         for tool_name in self.tools:
             if self.services and tool_name in self.services:
                 service = self.services[tool_name]
@@ -87,11 +103,13 @@ class LocalAgent(BaseAgent):
                         from AgentCrew.modules.memory.tool import (
                             register as register_memory,
                             adaptive_instruction_prompt,
+                            memory_instruction_prompt,
                         )
 
                         register_memory(
                             service, self.services.get("context_persistent", None), self
                         )
+                        self.tool_prompts.append(memory_instruction_prompt())
                         self.tool_prompts.append(adaptive_instruction_prompt())
                     elif tool_name == "clipboard":
                         from AgentCrew.modules.clipboard.tool import (
@@ -178,9 +196,21 @@ class LocalAgent(BaseAgent):
         Args:
             prompt: The system prompt
         """
-        self.system_prompt = prompt.replace(
-            "{current_date}", datetime.today().strftime("%A, %d/%m/%Y")
-        ).replace("{cwd}", os.getcwd())
+        self.system_prompt = prompt
+
+    def _parse_system_prompt(self, prompt: str) -> str:
+        """
+        Parse the system prompt to ensure it is in the correct format.
+
+        Args:
+            prompt: The system prompt
+        """
+        return (
+            prompt.replace("{current_date}", datetime.today().strftime("%A, %d/%m/%Y"))
+            .replace("{cwd}", os.getcwd())
+            .replace("{current_agent_name}", self.name)
+            .replace("{current_agent_description}", self.description)
+        )
 
     def set_custom_system_prompt(self, prompt: str):
         """
@@ -213,13 +243,13 @@ class LocalAgent(BaseAgent):
         if self.is_active:
             return True  # Already active
 
-        from AgentCrew.modules.mcpclient.manager import MCPSessionManager
-
         self.register_tools()
         self._register_tools_with_llm()
 
         # Reinitialize MCP session manager for the current agent
         if not self.is_remoting_mode:
+            from AgentCrew.modules.mcpclient.manager import MCPSessionManager
+
             mcp_manager = MCPSessionManager.get_instance()
             if mcp_manager.initialized:
                 mcp_manager.initialize_for_agent(self.name)
@@ -230,7 +260,7 @@ class LocalAgent(BaseAgent):
         if self.tool_prompts:
             system_prompt = system_prompt + "\n---\n\n" + "\n".join(self.tool_prompts)
 
-        self.llm.set_system_prompt(system_prompt)
+        self.llm.set_system_prompt(self._parse_system_prompt(system_prompt))
         self.llm.temperature = self.temperature if self.temperature is not None else 0.4
         self.is_active = True
         return True
@@ -247,7 +277,16 @@ class LocalAgent(BaseAgent):
 
         self._clear_tools_from_llm()
         self.tool_definitions = {}
+        self.tool_prompts = []
         self.is_active = False
+        self.is_tool_ready = True
+        # Reinitialize MCP session manager for the current agent
+        if not self.is_remoting_mode:
+            from AgentCrew.modules.mcpclient.manager import MCPSessionManager
+
+            mcp_manager = MCPSessionManager.get_instance()
+            if mcp_manager.initialized:
+                mcp_manager.cleanup_for_agent(self.name)
         return True
 
     def _register_tools_with_llm(self):
@@ -408,11 +447,12 @@ class LocalAgent(BaseAgent):
         _input_tokens_usage = 0
         _output_tokens_usage = 0
         # Ensure the first message is a system message with the agent's prompt
+        while not self.is_tool_ready:
+            await asyncio.sleep(0.2)
         if not messages:
             final_messages = list(self.history)
         else:
             final_messages = list(messages)
-            print(messages)
         if "context_persistent" in self.services and isinstance(
             self.services["context_persistent"], ContextPersistenceService
         ):
@@ -437,7 +477,17 @@ class LocalAgent(BaseAgent):
                         "content": [
                             {
                                 "type": "text",
-                                "text": f'MANDATORY: Check stored adaptive behaviors before responding. When "when...do..." conditions match, execute those behaviors immediately—they override default logic. Ask for clarification if uncertain which behaviors apply. List of adaptive behaviors: \n{adaptive_text}.\n END OF ADAPTABLE BEHAVIORS.\n\n',
+                                "text": f"""**ADAPTIVE BEHAVIORS CHECK**: Execute any matching active behaviors "when...do..." patterns immediately—these override standard responses.
+
+**Active Behaviors**: {adaptive_text}
+
+**Execution Rules**:
+• Match conditions → Execute behaviors
+• Unclear match → Ask for clarification  
+• Multiple matches → Apply most specific
+• No stored behaviors → Proceed with standard response
+
+**Note**: Use behaviors as response triggers only, not as memory search terms.""",
                             }
                         ],
                     }
@@ -475,8 +525,6 @@ class LocalAgent(BaseAgent):
 
         except GeneratorExit as e:
             logger.warning(f"Stream processing interrupted: {e}")
-        finally:
-            return
 
     def get_process_result(self):
         return (self.tool_uses, self.input_tokens_usage, self.output_tokens_usage)
