@@ -11,6 +11,7 @@ from AgentCrew.modules.agents import AgentManager
 from AgentCrew.modules.chat.file_handler import FileHandler
 from AgentCrew.modules.llm.message import MessageTransformer
 from AgentCrew.modules.config import ConfigManagement
+from AgentCrew.modules.voice import DeepInfraVoiceService, ElevenLabsVoiceService
 from AgentCrew.modules.memory import (
     BaseMemoryService,
     ContextPersistenceService,
@@ -69,6 +70,14 @@ class MessageHandler(Observable):
 
         self.conversation_manager.start_new_conversation()  # Initialize first conversation
 
+        # Voice integration
+        self.voice_service = None
+        # Check if voice service is available
+        if os.getenv("ELEVENLABS_API_KEY"):
+            self.voice_service = ElevenLabsVoiceService()
+        elif os.getenv("DEEPINFRA_API_KEY"):
+            self.voice_service = DeepInfraVoiceService()
+
     def _messages_append(self, message):
         """Append a message to the agent history and streamline messages."""
         self.agent.history.append(message)
@@ -191,6 +200,10 @@ class MessageHandler(Observable):
         start_thinking = False
         end_thinking = False
         has_stop_interupted = False
+        voice_sentence = "" if self._is_voice_enabled() else None
+        voice_id = (
+            self.voice_service.get_configured_voice_id() if self.voice_service else None
+        )
 
         # Create a reference to the streaming generator
         self.stream_generator = None
@@ -244,7 +257,27 @@ class MessageHandler(Observable):
                         # Delays it a bit when using without stream
                         time.sleep(0.5)
                     self._notify("response_chunk", (chunk_text, assistant_response))
+                    if voice_sentence is not None:
+                        voice_sentence += chunk_text
+                    if (
+                        voice_sentence
+                        and "\n" in voice_sentence.strip()
+                        and self.voice_service
+                    ):
+                        self.voice_service.text_to_speech_stream(
+                            voice_sentence.strip().partition("\n")[0], voice_id=voice_id
+                        )
+                        voice_sentence = None
 
+            # End thinking when break the response stream
+            if not end_thinking and start_thinking:
+                self._notify("thinking_completed", thinking_content)
+                end_thinking = True
+
+            if voice_sentence and voice_sentence.strip() and self.voice_service:
+                self.voice_service.text_to_speech_stream(
+                    voice_sentence.strip().partition("\n")[0], voice_id=voice_id
+                )
             # Handle tool use if needed
             if not has_stop_interupted and tool_uses and len(tool_uses) > 0:
                 # Add thinking content as a separate message if available
@@ -370,7 +403,7 @@ class MessageHandler(Observable):
                         user_input = user_message["content"]
 
                     try:
-                        await self.memory_service.store_conversation(
+                        self.memory_service.store_conversation(
                             user_input, assistant_response, self.agent.name
                         )
                     except Exception as e:
@@ -392,13 +425,32 @@ class MessageHandler(Observable):
                 )
                 self.current_user_input = None
                 self.current_user_input_idx = -1
+            if self.current_conversation_id and self.last_assisstant_response_idx >= 0:
+                # Get all messages added since the user input for this turn
+                current_provider = self.agent.get_provider()
+                messages_for_this_turn = MessageTransformer.standardize_messages(
+                    self.agent.history[self.last_assisstant_response_idx :],
+                    current_provider,
+                    self.agent.name,
+                )
+                if (
+                    messages_for_this_turn
+                ):  # Only save if there are messages for the turn
+                    self.persistent_service.append_conversation_messages(
+                        self.current_conversation_id, messages_for_this_turn
+                    )
+                    self._notify(
+                        "conversation_saved", {"id": self.current_conversation_id}
+                    )
+            self.last_assisstant_response_idx = len(self.agent.history)
+
             error_message = str(e)
             traceback_str = traceback.format_exc()
+            logger.error(f"{error_message} \n {traceback_str}")
             self._notify(
                 "error",
                 {
                     "message": error_message,
-                    "traceback": traceback_str,
                     "messages": self.agent.history,
                 },
             )
@@ -418,3 +470,13 @@ class MessageHandler(Observable):
     def delete_conversation_by_id(self, conversation_id: str) -> bool:
         """Deletes a conversation by its ID."""
         return self.conversation_manager.delete_conversation_by_id(conversation_id)
+
+    def _is_voice_enabled(self) -> bool:
+        """Check if voice is enabled in global settings."""
+        try:
+            config_management = ConfigManagement()
+            global_config = config_management.read_global_config_data()
+            return global_config.get("global_settings", {}).get("voice_enabled", False)
+        except Exception as e:
+            logger.warning(f"Failed to read voice_enabled setting: {e}")
+            return False
