@@ -5,7 +5,7 @@ from AgentCrew.modules.llm.model_registry import ModelRegistry
 import os
 import re
 
-COMPLETER_PATTERN = re.compile(r"[a-zA-Z0-9-_.]+")
+COMPLETER_PATTERN = re.compile(r"[a-zA-Z0-9-_.]*")
 
 
 class JumpCompleter(Completer):
@@ -161,6 +161,10 @@ class ChatCompleter(Completer):
                 "/consolidate",
                 "Consolidate conversation messages (usage: /consolidate [count])",
             ),
+            (
+                "/unconsolidate",
+                "Remove the last consolidated message (usage: /unconsolidate)",
+            ),
             (
                 "/jump",
                 "Jump to a previous conversation turn (usage: /jump <turn_number>)",
